# Generated by roxygen2: do not edit by hand

S3method(autoplot,DTRreg)
S3method(coef,DTRreg)
S3method(confint,DTRreg)
S3method(plot,DTRreg)
S3method(predict,DTRreg)
S3method(print,DTRreg)
S3method(summary,DTRreg)
export(DTRreg)
export(DWSurv)
export(chooseM)
import(R6)
import(ggplot2)
importFrom(R6,is.R6)
importFrom(graphics,hist)
importFrom(nnet,multinom)
importFrom(stats,Gamma)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,confint)
importFrom(stats,family)
importFrom(stats,fitted)
importFrom(stats,gaussian)
importFrom(stats,get_all_vars)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.pass)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(stats,update.formula)
importFrom(stats,var)
importFrom(utils,capture.output)
importFrom(utils,tail)
